program estimate_transition

!Compile
!mpiifort params.f90 mini.f90 estimate_transition.f90 -o estimate_transition
!mpiexec -np 1 ./estimate_transition

use params
use mini

implicit none
include 'mpif.h'

!number of columns for data file
integer, parameter :: columns=9     !age, education, expense1, expense2, dead, weight, health1, health2

!variables to count rows in data
integer:: rows, ios, dummy
integer :: rc   

!random number
double precision :: rn 

!data variable
double precision, allocatable, dimension(:,:) :: data, data_temp, dist, data_all

!counters
integer :: count0, cnt_unique, count_alive
integer :: rcc

!population in current period and next period given age and health
double precision, dimension(ne,nj,nh) :: pop0, pop1

!storing variables
double precision :: temp, store

!check variables
integer :: check1, check2

!misc variables
double precision :: alive
double precision :: expense_temp(nh)

integer :: iter_adjust
double precision, dimension(ne,nh-1) :: adjust_pi_h
double precision :: adjust_psi

integer :: cnt_all

call MPI_INIT (ierr)
call MPI_COMM_SIZE (mpi_comm_world, nproc, ierr)
call MPI_COMM_RANK (mpi_comm_world, rank, ierr)


call cpu_time(codestart)

    open (unit=1,file='input_survival_probabilities.txt')
        do jc=1,nj1
        read(1,*)psibyage_census(jc)
        end do
    close(1)
        
!count number of rows from input_raw_data.txt
rows=0

    open (1, file='input_raw_data.txt')
        do
        read (1,*,iostat=ios)dummy
           if (ios/=0) exit     
        rows=rows+1
        end do
    close(1)
    
    if ( rank .eq. 0 ) then
    print*,'rows in input_raw_data.txt ',rows
    end if

!allocate rows and columns to data

    allocate(data(rows,columns))   
    allocate(data_all(rows+ne*nj*4,columns))   
    
    data=zero
    data_all=zero
    
    !read data

    open (unit=1,file='input_raw_data.txt')
        do rc=1,rows
        read(1,*)data(rc,1:6)
        end do
    close(1)

    !add small random numbers to expense 1 and expense 2

    do rc=1,rows        

        call random_number(rn)
        
    data(rc,3)=(data(rc,3)+rn*small)/y_2010
        
        call random_number(rn)        
            
    data(rc,4)=(data(rc,4)+rn*small)/y_2010
        
    end do

    cnt_all=0
    
        do ec=1,ne
        do jc=1,nj
        
        count0=0
        
            do rc=1,rows 
            
                if ( int(data(rc,1)) .eq. jc+19 .and. ec .eq. int(data(rc,2))  ) then                
                count0=count0+1
                end if
                
            end do
                    
            allocate(data_temp(count0+4,columns))
            
            data_temp=zero            
            count0=0
        
                do rc=1,rows 
            
                    if ( int(data(rc,1)) .eq. jc+19 .and. ec .eq. int(data(rc,2)) ) then                
                    count0=count0+1
                    data_temp(count0,1:6)=data(rc,1:6)
                    data_temp(count0,9)=data(rc,6)                    
                    end if
                
                end do
            
            !given age and education, normalize weights to sum to one
            data_temp(:,6)=data_temp(:,6)/sum(data_temp(:,6))
    
                call sort(data_temp(1:count0,:),count0,columns,3)    
                call unique(data_temp(1:count0,:),count0,columns,3,cnt_unique)                
            
                if ( cnt_unique .ne. count0 .and. rank .eq. 0 ) then
                print*,'number of unique rows lt count0 ',cnt_unique,count0
                    call exit
                end if  

            allocate(dist(count0,2))
 
            dist(1:count0,1)=data_temp(1:count0,3)
            dist(1:count0,2)=data_temp(1:count0,6)                    
                
                call percentile (pc1,dist,count0,perc1(ec,jc))                
                call percentile (pc2,dist,count0,perc2(ec,jc))
        
            deallocate(dist)
                        
            temp=zero
            check1=0
            check2=0
    
                do rc=1,count0
        
                temp=temp+data_temp(rc,6)
        
                    if ( temp .ge. pc1 .and. check1 .eq. 0 ) then
            
                    check1=1
                    store=data_temp(rc,6)
                    data_temp(rc,6)=temp-pc1
                    data_temp(count0+1,6)=store-data_temp(rc,6)

                    data_temp(count0+1,9)=data_temp(rc,9)*(1-(temp-pc1)/store)                
                    data_temp(rc,9)=data_temp(rc,9)*(temp-pc1)/store

                    
                        if ( perc1(ec,jc) .ne. data_temp(rc,3) ) then
                        print*,'problem: perc1(ec,jc) .ne. data_temp(rc,3)'
                            call exit
                        end if
                                                                                 
                        call random_number(rn)
                
                    data_temp(rc,3)=perc1(ec,jc)+rn*small  
                    data_temp(count0+1,1:5)=data_temp(rc,1:5)                   
                    data_temp(count0+1,3)=perc1(ec,jc)-rn*small
                    data_temp(count0+1,4)=data_temp(rc,4)+rn*small
            
                    end if

                    if ( temp .ge. pc2 .and. check2 .eq. 0 ) then
            
                    check2=1
                    store=data_temp(rc,6)
                    data_temp(rc,6)=temp-pc2
                    data_temp(count0+2,6)=store-data_temp(rc,6)

                    data_temp(count0+2,9)=data_temp(rc,9)*(1-(temp-pc2)/store)                
                    data_temp(rc,9)=data_temp(rc,9)*(temp-pc2)/store
                    
                        if ( perc2(ec,jc) .ne. data_temp(rc,3) ) then
                        print*,'problem: perc2(ec,jc) .ne. data_temp(rc,3)'
                            call exit
                        end if
            
                        call random_number(rn)
                
                    data_temp(rc,3)=perc2(ec,jc)+rn*small  
                    data_temp(count0+2,1:5)=data_temp(rc,1:5)                   
                    data_temp(count0+2,3)=perc2(ec,jc)-rn*small
                    data_temp(count0+2,4)=data_temp(rc,4)+rn*small
                    
                    end if
                        
                end do
                    
            pop0(ec,jc,:)=zero
                     
                do rc=1,count0+2
            
                    if ( data_temp(rc,3) .le. perc1(ec,jc) ) then
                        
                    data_temp(rc,7)=1
                    pop0(ec,jc,1)=pop0(ec,jc,1)+data_temp(rc,6)
                        
                    else if ( data_temp(rc,3) .gt. perc1(ec,jc) .and. data_temp(rc,3) .le. perc2(ec,jc) ) then
                        
                    data_temp(rc,7)=2
                    pop0(ec,jc,2)=pop0(ec,jc,2)+data_temp(rc,6)                        
                        
                    else if ( data_temp(rc,3) .gt. perc2(ec,jc) ) then
                        
                    data_temp(rc,7)=3
                    pop0(ec,jc,3)=pop0(ec,jc,3)+data_temp(rc,6)                                                
                        
                    end if
        
                end do   
                    
            !For expense2
            
            count_alive=0
            
                do rc=1,count0+2
                
                    if ( int(data_temp(rc,5)) .eq. 0 ) then            
                    count_alive=count_alive+1
                    end if
               
                end do               
                                                       
                call sort(data_temp(1:count0+2,:),count0+2,columns,4)    
                call unique(data_temp(1:count0+2,:),count0+2,columns,4,cnt_unique)                
                            
                if ( cnt_unique .ne. count0+2 .and. rank .eq. 0 ) then
                print*,'number of unique rows lt count0 ',cnt_unique,count0+2
                    call exit
                end if  

            allocate(dist(count_alive,2))
 
            count_alive=0
            
                do rc=1,count0+2
               
                    if ( int(data_temp(rc,5)) .eq. 0 ) then            
                    count_alive=count_alive+1
                    dist(count_alive,1)=data_temp(rc,4)
                    dist(count_alive,2)=data_temp(rc,6)                    
                    end if
               
                end do               

            dist(:,2)=dist(:,2)/sum(dist(:,2))
            
                call percentile (pc1,dist,count_alive,perc1p(ec,jc))                
                call percentile (pc2,dist,count_alive,perc2p(ec,jc))        
                    
            temp=zero
            check1=0
            check2=0
            rcc=0
                
                do rc=1,count_alive
 
                temp=temp+dist(rc,2)

                    if ( int(data_temp(rc,5)) .eq. 1 ) then            
                    rcc=rcc+1
                    end if
        
                    if ( temp .ge. pc1 .and. check1 .eq. 0 ) then
            
                    check1=1
                    store=dist(rc,2)
                    data_temp(count0+3,6)=data_temp(rc+rcc,6)*(1-(temp-pc1)/store)                 
                    data_temp(rc+rcc,6)=data_temp(rc+rcc,6)*(temp-pc1)/store

                    data_temp(count0+3,9)=data_temp(rc+rcc,9)*(1-(temp-pc1)/store)                
                    data_temp(rc+rcc,9)=data_temp(rc+rcc,9)*(temp-pc1)/store
                    
                        if ( perc1p(ec,jc) .ne. data_temp(rc+rcc,4) ) then
                        print*,'problem: perc1p(ec,jc) .ne. data_temp(rc+rcc,4)'
                            call exit
                        end if
                                                                                 
                        call random_number(rn)
                
                    data_temp(rc+rcc,4)=perc1p(ec,jc)+rn*small  
                    data_temp(count0+3,1:5)=data_temp(rc+rcc,1:5)                   
                    data_temp(count0+3,7)=data_temp(rc+rcc,7)                                           
                    data_temp(count0+3,4)=perc1p(ec,jc)-rn*small
                                    
                    end if

                    if ( temp .ge. pc2 .and. check2 .eq. 0 ) then
            
                    check2=1
                    store=dist(rc,2)
                    data_temp(count0+4,6)=data_temp(rc+rcc,6)*(1-(temp-pc2)/store)                          
                    data_temp(rc+rcc,6)=data_temp(rc+rcc,6)*(temp-pc2)/store

                    data_temp(count0+4,9)=data_temp(rc+rcc,9)*(1-(temp-pc2)/store)                
                    data_temp(rc+rcc,9)=data_temp(rc+rcc,9)*(temp-pc2)/store
                    
                        if ( perc2p(ec,jc) .ne. data_temp(rc+rcc,4) ) then
                        print*,'problem: perc2p(ec,jc) .ne. data_temp(rc+rcc,4)'
                            call exit
                        end if
            
                        call random_number(rn)
                            
                    data_temp(rc+rcc,4)=perc2p(ec,jc)+rn*small  
                    data_temp(count0+4,1:5)=data_temp(rc+rcc,1:5)                   
                    data_temp(count0+4,7)=data_temp(rc+rcc,7)                                           
                    data_temp(count0+4,4)=perc2p(ec,jc)-rn*small                        

                    
                    end if
                        
                end do
                    
                if ( check1 .eq. 0 .or. check2 .eq. 0 ) then
                print*,'check1/check2 .eq. 0 '
                    call exit
                end if
                    
                deallocate(dist)   

            pop1(ec,jc,:)=zero
                     
                do rc=1,count0+4
            
                    if ( data_temp(rc,4) .le. perc1p(ec,jc) .and. int(data_temp(rc,5)) .eq. 0  ) then
                        
                    data_temp(rc,8)=1
                    pop1(ec,jc,1)=pop1(ec,jc,1)+data_temp(rc,6)
                        
                    else if ( data_temp(rc,4) .gt. perc1p(ec,jc) .and. data_temp(rc,4) .le. perc2p(ec,jc) .and. int(data_temp(rc,5)) .eq. 0  ) then
                        
                    data_temp(rc,8)=2
                    pop1(ec,jc,2)=pop1(ec,jc,2)+data_temp(rc,6)                        
                        
                    else if ( data_temp(rc,4) .gt. perc2p(ec,jc) .and. int(data_temp(rc,5)) .eq. 0  ) then
                        
                    data_temp(rc,8)=3
                    pop1(ec,jc,3)=pop1(ec,jc,3)+data_temp(rc,6)                                                
                        
                    end if
        
                end do

            pop1(ec,jc,:)=pop1(ec,jc,:)/sum(pop1(ec,jc,:))

            !estimate death probablities
            death(ec,jc,:)=zero
                    
                do rc=1,count0+4

                    do hc=1,nh
                    
                        if ( int(data_temp(rc,7)) .eq. hc .and. int(data_temp(rc,5)) .eq. 1 ) then
                        death(ec,jc,hc)=death(ec,jc,hc)+data_temp(rc,6)                    
                        end if
                        
                    end do
                    
                end do

            death(ec,jc,:)=death(ec,jc,:)/pop0(ec,jc,:)          
            psi(ec,jc,:)=1-death(ec,jc,:)
        
            pi_h(ec,jc,:,:)=zero

                do rc=1,count0+4
                        
                    do hc=1,nh
                    do hcc=1,nh
                    
                        if ( int(data_temp(rc,7)) .eq. hc .and. int(data_temp(rc,8)) .eq. hcc .and. int(data_temp(rc,5)) .eq. 0 ) then
                        pi_h(ec,jc,hcc,hc)=pi_h(ec,jc,hcc,hc)+data_temp(rc,6)
                        end if
                            
                    end do
                    end do
                        
                end do

                do hc=1,nh
                
                    if ( sum(pi_h(ec,jc,:,hc)) .gt. zero ) then
                    pi_h(ec,jc,:,hc)=pi_h(ec,jc,:,hc)/sum(pi_h(ec,jc,:,hc))
                    end if
                    
                end do
                    
            med_expense(ec,jc,:)=zero
            expense_temp=zero
            
                do rc=1,count0+4
                do hc=1,nh

                    if ( int(data_temp(rc,7)) .eq. hc  ) then
                    med_expense(ec,jc,hc)=med_expense(ec,jc,hc)+data_temp(rc,3)*data_temp(rc,6)
                    expense_temp(hc)=expense_temp(hc)+data_temp(rc,6)                    
                    end if
                    
                end do
                end do
                
            med_expense(ec,jc,:)=med_expense(ec,jc,:)/expense_temp(:)            
                
                do rc=1,count0+4
                cnt_all=cnt_all+1
                data_all(cnt_all,:)=data_temp(rc,:)
                end do
            
            deallocate(data_temp)            
            
         end do
         end do

        do cnt_all=1,rows+ne*nj*4
        
            if ( int(data_all(cnt_all,8)) .eq. 0 ) then
            data_all(cnt_all,8)=4
            end if
            
        end do
            
        open (1, file='output_raw_data.txt')
            do cnt_all=1,rows+ne*nj*4
            write(1,'(300F12.4)')data_all(cnt_all,1:2),data_all(cnt_all,7:9)      
!            write(1,'(I2,I2,300F12.9,300F12.9,I2,300F12.9,I2,I2,300F12.4)')int(data_all(cnt_all,1)),int(data_all(cnt_all,2)),data_all(cnt_all,3),data_all(cnt_all,4),&
!            int(data_all(cnt_all,5)),data_all(cnt_all,6),int(data_all(cnt_all,7)),int(data_all(cnt_all,8)),data_all(cnt_all,9)
            end do
        close(1)
        
        
        deallocate(data_all)

Nu=zero
Nu(:,1,:)=pop0(:,1,:)
Nu(1,1,:)=pop0(1,1,:)*nc_frac
Nu(2,1,:)=pop0(2,1,:)*(1-nc_frac)
    
    do ec=1,ne
    do jc=1,nj-1
    do hc=1,nh
        do hcc=1,nh
        Nu(ec,jc+1,hcc)=Nu(ec,jc+1,hcc)+(psi(ec,jc,hc)*Nu(ec,jc,hc)*pi_h(ec,jc,hcc,hc))/(1+nn)	
        end do
    end do
    end do
    end do

      print*,'dependency ratio ',sum(Nu(:,47:nj,:))/sum(Nu(:,1:46,:))
      print*,'nc_frac ',sum(Nu(1,:,:))/sum(Nu)
  
psibyage_meps=zero

    do jc=1,nj-1
    psibyage_meps(jc)=(1+nn)*sum(Nu(:,jc+1,:))/sum(Nu(:,jc,:))
    end do    
    
    
max_error=zero

    do ec=1,ne
    do jc=1,nj
    
    Nu(ec,jc,:)=Nu(ec,jc,:)/sum(Nu(ec,jc,:))
    
        if ( abs(Nu(ec,jc,1)-pc1) .gt. max_error(ec,1) ) then
        max_error(ec,1)=abs(Nu(ec,jc,1)-pc1)
        end if

        if ( abs(Nu(ec,jc,2)-(pc2-pc1)) .gt. max_error(ec,2) ) then
        max_error(ec,2)=abs(Nu(ec,jc,2)-(pc2-pc1))
        end if

        if ( abs(Nu(ec,jc,3)-(1-pc2)) .gt. max_error(ec,3) ) then
        max_error(ec,3)=abs(Nu(ec,jc,3)-(1-pc2))
        end if
        
    end do
    end do
    
    
        if ( rank .eq. 0 ) then

        print*,'max_error '
        
            do ec=1,ne
            write(6,'(300F12.9)')max_error(ec,:)*100
            end do            
                    
            open(unit=1,file='output_transition_probalities.txt')
            write(1,*)'age edu health prob1 prob2 prob3 death'
                do jc=1,nj
                do ec=1,ne
                do hc=1,nh
                write(1,'(I2,I2,I2,300F12.9)')19+jc,ec,hc,pi_h(ec,jc,:,hc),death(ec,jc,hc)
                end do
                end do
                end do
            close(1)
        
            open(unit=2,file='output_medical_expense.txt')
            write(2,*)'age nc_l nc_h nc_c nc_p1 nc_p2 nc_p1p nc_p2p c_l c_h c_c c_p1 c_p2 c_p1p c_p2p'            
                do jc=1,nj
                write(2,'(I2,300F12.8)')19+jc,med_expense(1,jc,:),perc1(1,jc),perc2(1,jc),perc1p(1,jc),perc2p(1,jc),&
                med_expense(2,jc,:),perc1(2,jc),perc2(2,jc),perc1p(2,jc),perc2p(2,jc)
                end do
            close(2)
             
        end if
        
    !computing the trend
    
    do jc=1,nj
    age(jc)=19+jc
    end do

    do jc=1,nj1
    age_trend(jc)=19+jc
    end do
    
    do ec=1,ne
    do hc=1,nh
        do hcc=1,nh    
        
!            call polynomial (age,pi_h(ec,:,hcc,hc),nj,poly_power,age_trend,pi_h_trend(ec,:,hcc,hc),nj1)
            
        end do        
    
!        call polynomial (age,psi(ec,:,hc),nj,poly_power,age_trend,psi_trend(ec,:,hc),nj1)
        call polynomial (age,med_expense(ec,:,hc),nj,poly_power,age_trend,med_expense_trend(ec,:,hc),nj1)
        
    end do    
    end do

    
    open(unit=1,file='input_health_probabilities_reg.txt')
        do jc=1,nj1
        do ec=1,ne
        do hc=1,nh
        read(1,*)pi_h_trend(ec,jc,:,hc),psi_trend(ec,jc,hc)
        end do
        end do
        end do
    close(1)
    
    do ec=1,ne
    do jc=1,nj1
    do hc=1,nh
    
        do hcc=1,nh
        
            if ( pi_h_trend(ec,jc,hcc,hc) .lt. zero ) then
            print*,'problem estimate_transition.f90'
                call exit
            end if
            
        end do
        
    pi_h_trend(ec,jc,:,hc)=pi_h_trend(ec,jc,:,hc)/sum(pi_h_trend(ec,jc,:,hc))    
    
        if ( psi_trend(ec,jc,hc) .gt. one ) then
        print*,'problem estimate_transition.f90'
            call exit
        end if
        
    end do
    end do
    end do
    
Nu_trend=zero
!Nu_trend(:,1,:)=pop0(:,1,:)
Nu_trend(1,1,:)=pop0(1,1,:)*nc_frac
Nu_trend(2,1,:)=pop0(2,1,:)*(1-nc_frac)

    do ec=1,ne
    do jc=1,nj1-1
    do hc=1,nh
    
        do hcc=1,nh
        Nu_trend(ec,jc+1,hcc)=Nu_trend(ec,jc+1,hcc)+(psi_trend(ec,jc,hc)*Nu_trend(ec,jc,hc)*pi_h_trend(ec,jc,hcc,hc))/(1+nn)	
        end do
        
    end do
    end do
    end do

psibyage_trend=zero

    do jc=1,nj1-1
    psibyage_trend(jc)=(1+nn)*sum(Nu_trend(:,jc+1,:))/sum(Nu_trend(:,jc,:))
    end do    
    
    print*,'dependency ratio trend',sum(Nu_trend(:,47:nj1,:))/sum(Nu_trend(:,1:46,:))
    print*,'nc_frac trend',sum(Nu_trend(1,:,:))/sum(Nu_trend)
    
max_error_trend=zero
    
    do ec=1,ne
    do jc=1,nj1
    
    Nu_trend(ec,jc,:)=Nu_trend(ec,jc,:)/sum(Nu_trend(ec,jc,:))
    
        if ( abs(Nu_trend(ec,jc,1)-pc1) .gt. max_error_trend(ec,1) ) then
        max_error_trend(ec,1)=abs(Nu_trend(ec,jc,1)-pc1)
        end if

        if ( abs(Nu_trend(ec,jc,2)-(pc2-pc1)) .gt. max_error_trend(ec,2) ) then
        max_error_trend(ec,2)=abs(Nu_trend(ec,jc,2)-(pc2-pc1))
        end if

        if ( abs(Nu_trend(ec,jc,3)-(1-pc2)) .gt. max_error_trend(ec,3) ) then
        max_error_trend(ec,3)=abs(Nu_trend(ec,jc,3)-(1-pc2))
        end if

    end do
    end do
    
    print*,'max_error_trend'
    
        do ec=1,ne
        write(6,'(300F12.7)')max_error_trend(ec,:)*100
        end do
            
            open(unit=1,file='output_transition_probalities_trend.txt')
            write(1,*)'age edu health prob1 prob2 prob3 death'
                do jc=1,nj1
                do ec=1,ne
                do hc=1,nh
                write(1,'(I2,I2,I2,300F12.9)')19+jc,ec,hc,pi_h_trend(ec,jc,:,hc),max(1-psi_trend(ec,jc,hc),zero)
                end do
                end do
                end do
            close(1)

            open(unit=1,file='output_medical_expense_trend.txt')
            write(1,*)'age nc_l_t nc_h_t nc_c_t c_l_t c_h_t c_c_t'            
                do jc=1,nj1
                write(1,'(I2,300F12.9)')19+jc,med_expense_trend(1,jc,:),med_expense_trend(2,jc,:)
                end do
            close(1)
            
            
!computing average med expense given age and health
med_expense_avg=zero

    do jc=1,nj
    do hc=1,nh
    med_expense_avg(jc,hc)=sum(med_expense(:,jc,hc))/ne
    end do
    end do
    
    do hc=1,nh
    
        call polynomial (age,med_expense_avg(:,hc),nj,poly_power,age_trend,med_expense_avg_trend(:,hc),nj1)
        
    end do

        open(unit=1,file='output_medical_expense_avg.txt')
        write(1,*)'age l h c l_t h_t c_t'          
            do jc=1,nj1
            write(1,'(I2,300F12.9)')19+jc,med_expense_avg(jc,:),med_expense_avg_trend(jc,:)
            end do
        close(1)
    
!initial guess for scaling
pi_h_adjusted=pi_h_trend
psi_adjusted=psi_trend
!Nu_adjusted(:,1,:)=pop0(:,1,:)
Nu_adjusted(1,1,:)=pop0(1,1,:)*nc_frac
Nu_adjusted(2,1,:)=pop0(2,1,:)*(1-nc_frac)

    do jc=1,nj1-1
    
    !scaling pramaeters
    adjust_pi_h=one
    adjust_psi=one

        do ec=1,ne
        do hc=1,nh    
        do hcc=1,nh
        Nu_adjusted(ec,jc+1,hcc)=Nu_adjusted(ec,jc+1,hcc)+(psi_adjusted(ec,jc,hc)*Nu_adjusted(ec,jc,hc)*pi_h_adjusted(ec,jc,hcc,hc))/(1+nn)	
        end do
        end do
        end do
    
        do iter_adjust=1,10000
            
        Nu_adjusted(:,jc+1,:)=zero

            do ec=1,ne
            do hc=1,nh    
            do hcc=1,nh
            Nu_adjusted(ec,jc+1,hcc)=Nu_adjusted(ec,jc+1,hcc)+(psi_adjusted(ec,jc,hc)*Nu_adjusted(ec,jc,hc)*pi_h_adjusted(ec,jc,hcc,hc))/(1+nn)	
            end do
            end do
            end do

            do ec=1,ne

            hcc=2
            
            adjust_pi_h(ec,1)=adjust_pi_h(ec,1)*((pc2-pc1)/(Nu_adjusted(ec,jc+1,hcc)/sum(Nu_adjusted(ec,jc+1,:))))**.01            
            pi_h_adjusted(ec,jc,hcc,:)=pi_h_trend(ec,jc,hcc,:)*adjust_pi_h(ec,1)

            hcc=nh
            adjust_pi_h(ec,2)=adjust_pi_h(ec,2)*((1-pc2)/(Nu_adjusted(ec,jc+1,hcc)/sum(Nu_adjusted(ec,jc+1,:))))**.01
            pi_h_adjusted(ec,jc,hcc,:)=pi_h_trend(ec,jc,hcc,:)*adjust_pi_h(ec,2)        

            end do

            do ec=1,ne
            do hc=1,nh
            pi_h_adjusted(ec,jc,1,hc)=1-sum(pi_h_adjusted(ec,jc,2:nh,hc))
                
                if ( pi_h_adjusted(ec,jc,1,hc) .lt.zero ) then
                print*,'negative prob in adjustment estimate_transition.f90 '
                    call exit
                end if
                
            end do
            end do

        psibyage_adjusted(jc)=(1+nn)*sum(Nu_adjusted(:,jc+1,:))/sum(Nu_adjusted(:,jc,:))
        adjust_psi=adjust_psi*(psibyage_census(jc)/psibyage_adjusted(jc))**.01
        psi_adjusted(:,jc,:)=min(psi_trend(:,jc,:)*adjust_psi,one)     
            
        end do
        
    end do

    print*,'dependency ratio adjusted',sum(Nu_adjusted(:,47:nj1,:))/sum(Nu_adjusted(:,1:46,:))
    print*,'nc_frac adjusted ',sum(Nu_adjusted(1,:,:))/sum(Nu_adjusted)
    
max_error_adjusted=zero
    
    do ec=1,ne
    do jc=1,nj1
    
    Nu_adjusted(ec,jc,:)=Nu_adjusted(ec,jc,:)/sum(Nu_adjusted(ec,jc,:))
    
        if ( abs(Nu_adjusted(ec,jc,1)-pc1) .gt. max_error_adjusted(ec,1) ) then
        max_error_adjusted(ec,1)=abs(Nu_adjusted(ec,jc,1)-pc1)
        end if

        if ( abs(Nu_adjusted(ec,jc,2)-(pc2-pc1)) .gt. max_error_adjusted(ec,2) ) then
        max_error_adjusted(ec,2)=abs(Nu_adjusted(ec,jc,2)-(pc2-pc1))
        end if

        if ( abs(Nu_adjusted(ec,jc,3)-(1-pc2)) .gt. max_error_adjusted(ec,3) ) then
        max_error_adjusted(ec,3)=abs(Nu_adjusted(ec,jc,3)-(1-pc2))
        end if

    end do
    end do
        
    print*,'max_error_adjusted'
    
        do ec=1,ne
        write(6,'(300F12.9)')max_error_adjusted(ec,:)*100
        end do

        if ( rank .eq. 0 ) then
        
            open(unit=1,file='output_transition_probalities_adjusted.txt')
            write(1,*)'age edu health prob1 prob2 prob3 death'
                do jc=1,nj1
                do ec=1,ne
                do hc=1,nh
                write(1,'(I2,I2,I2,300F12.9)')19+jc,ec,hc,pi_h_adjusted(ec,jc,:,hc),one-psi_adjusted(ec,jc,hc)
                end do
                end do
                end do
            close(1)
        
            open (unit=1,file='output_psi_by_age.txt')
            write(1,*)'age census meps trend adjusted '
                do jc=1,nj1-1
                write(1,'(I2,300F12.9)')19+jc,psibyage_census(jc),psibyage_meps(jc),psibyage_trend(jc),psibyage_adjusted(jc)
                end do
            close(1)

            open (unit=1,file='input_health_probabilities.txt')
                do jc=1,nj1-1
                do ec=1,ne
                do hc=1,nh
                write(1,'(300F12.9)')pi_h_adjusted(ec,jc,:,hc),psi_adjusted(ec,jc,hc)
                end do
                end do
                end do
            close(1)

            open(unit=1,file='input_medical_expense.txt')
                do hc=1,nh
                write(1,'(300F12.9)')max(med_expense_avg_trend(:,hc),zero)
                end do
            close(1)

            
        end if
    
call MPI_FINALIZE(ierr)
call cpu_time(codefinish)

	codetime=codefinish-codestart
	if ( rank .eq. 0 ) then
	PRINT*,'code time=',codetime/60.
	end if

end program estimate_transition