module mini
use params

contains

subroutine polynomial (x,y,n_rows,n_order,xtrend,ytrend,n_rows1)
implicit none

integer, intent(in) :: n_rows, n_order, n_rows1
double precision, dimension(n_rows), intent(in) :: x, y
double precision, dimension(n_rows1), intent(in) :: xtrend
double precision, dimension(n_rows1), intent(out) :: ytrend

double precision, dimension(n_rows,n_order) :: x_ols
double precision, dimension(n_order) :: beta_ols
double precision, dimension(n_order,n_order) :: xtransposex, xtransposex_inv
integer :: info
integer :: i, rc 

    do i=1,n_order
    x_ols(:,i)=x(:)**(i-one)
    end do
    
xtransposex=matmul(transpose(x_ols),x_ols)
    
   call INVERT(xtransposex,xtransposex_inv,n_order,info)
    
    if ( info .ne. 0 ) then
    print*,'matrix inversion failed '
        call exit
    end if
    
    beta_ols=matmul(matmul(xtransposex_inv,transpose(x_ols)),log(y))

ytrend=0
    
    do rc=1,n_rows1            
    
        do i=1,n_order
        ytrend(rc)=ytrend(rc)+beta_ols(i)*(xtrend(rc)**(i-one))
        end do
        
    ytrend(rc)=exp(ytrend(rc))

    end do
    
end subroutine polynomial

subroutine percentile (p1,x,nrows,perc)
implicit none

double precision, intent(in) :: p1
integer, intent(in) :: nrows
double precision, intent(in) :: x(nrows,2)
double precision, intent(out) :: perc

integer :: check
double precision :: temp
integer :: i
 
i=1
    check=0
temp=0
    
        do while ( i .le. nrows .and. check .eq. 0 )

        temp=temp+x(i,2)
        
            if ( temp .ge. p1 .and. check .eq. 0 ) then
            check=1
            perc=x(i,1)
            end if
            
	i=i+1
	
	end do
	
	if ( check .eq. 0 ) then
	print*,'did not find p1 in mini.f90 '
	end if
	
end subroutine percentile

subroutine unique(x,nrows,ncols,uniquecol,cnt)
implicit none

double precision, dimension(nrows,ncols), intent(in) :: x
integer, intent(in) :: nrows, ncols, uniquecol
integer, intent(out) :: cnt
integer :: i

    cnt=1

    do i=2,nrows
    
        if ( x(i,uniquecol) .ne. x(i-1,uniquecol) ) then
        cnt=cnt+1
        end if	
        
        if ( x(i,uniquecol) .lt. x(i-1,uniquecol) ) then
        print*,'sort must be wrong mini.f90'
            call exit
        end if
        
    end do

end subroutine unique

subroutine sort(x,nrows,ncols,sortcol)
implicit none

integer, intent(in) :: nrows, ncols, sortcol
double precision, dimension(nrows,ncols), intent(inout) :: x

integer :: i, location
double precision :: y(nrows)

y(:)=x(:,sortcol)

    do i=1,nrows-1
    
    location=findmin(y,i,nrows)
    
            call swap(x(i,:),x(location,:),ncols)
            
    y(:)=x(:,sortcol)
    
    end do

end subroutine sort

function findmin (x,beg,en)
implicit none
double precision, dimension(:), intent(in) :: x
integer, intent(in) :: beg, en

double precision :: minimum
integer :: i, findmin

minimum=x(beg)
findmin=beg

	do i=beg+1,en
            if ( x(i) .le. minimum ) then
            minimum=x(i)
            findmin=i
            end if
	end do

end function findmin

subroutine swap(a,b,ncols)
implicit none
integer, intent(in) :: ncols
double precision, intent(inout) :: a(ncols), b(ncols)
double precision :: temp(ncols)

temp=a
a=b
b=temp
end subroutine swap

SUBROUTINE INVERT(AZ,AZV,NZ,INFO)
IMPLICIT double precision (A-H,O-Z)
DIMENSION AZ(NZ,NZ),AZV(NZ,NZ),AZZ(NZ,NZ)
integer :: INFO

INF0=0

      DO 20 I=1,NZ
        DO 10 J=1,NZ
        AZZ(I,J) = AZ(I,J)
        AZV(I,J) = 0.0

    10 CONTINUE
      AZV(I,I) = 1.0
      
   20 CONTINUE
   
      DO 80 I1=1,NZ
      
        IF(DABS(AZZ(I1,I1)).GT.1.D-12) GO TO 50
            IF(I1.EQ.NZ) GO TO 50
            I1P = I1+1
            IP1V=I1
            PMAX = DABS(AZZ(I1,I1))
            DO 30 I2=I1P,NZ
                IF(DABS(AZZ(I2,I1)).LT. PMAX) GO TO 30
                PMAX = DABS(AZZ(I2,I1))
                IP1V=I2
   30 CONTINUE
   
      IF(IP1V.EQ.I1) GO TO 50
      DO 40 I3=1,NZ
      AT1 = AZZ(I1,I3)
      AT2 = AZV(I1,I3)
      AZZ(I1,I3)=AZZ(IP1V,I3)
      AZV(I1,I3)=AZV(IP1V,I3)
      AZZ(IP1V,I3) = AT1
      AZV(IP1V,I3) = AT2
   40 CONTINUE
   50 CONTINUE
      AP1V = AZZ(I1,I1)
	IF(DABS(AP1V).LT.1.D-13)then 
	WRITE(6,*)'MATRIX IS SINGULAR'
!        INFO=1
            call exit
        end if
	IF(DABS(AP1V).LT.1.D-13)RETURN
      DO 55 J1=1,NZ
      AZZ(I1,J1) = AZZ(I1,J1)/AP1V
      AZV(I1,J1) = AZV(I1,J1)/AP1V
   55 CONTINUE
      DO 70 I4=1,NZ
      IF(I4.EQ.I1)GO TO 70
      IF(DABS(AZZ(I4,I1)).LT.1.D-12)GO TO 70
      AP2V=AZZ(I4,I1)
      DO 60 J2=1,NZ
      IF(DABS(AZZ(I1,J2)).LT.1.D-12)GO TO 65
      AZZ(I4,J2)=AZZ(I4,J2)-AP2V*AZZ(I1,J2)
   65 CONTINUE
      AZV(I4,J2)=AZV(I4,J2)-AP2V*AZV(I1,J2)
   60 CONTINUE
   70 CONTINUE
   80 CONTINUE
      RETURN
      
END SUBROUTINE INVERT



END MODULE mini