subroutine housess
! This Subroutine solves the household problem for all types in the steady state
use params
use mini

implicit none
include 'mpif.h'
double precision, allocatable :: cmpi(:), lmpi(:), ampi(:), vmpi(:), medicaidmpi(:), netmpi(:)
double precision, allocatable :: crecmpi(:), lrecmpi(:), arecmpi(:), vrecmpi(:), medicaidrecmpi(:), netrecmpi(:)
integer(kind=1), allocatable :: dfmpi(:), dgmpi(:), dmmpi(:)
integer(kind=1), allocatable :: dfrecmpi(:), dgrecmpi(:), dmrecmpi(:)

!variables to compute foodstamp value
double precision :: VM(nh,ns), vals(2), temp(nh)
integer :: ih,is,inds(2)
double precision :: vnotpay

!accuracy of minimization
double precision, parameter :: xacc=1e-11

!for mpi
integer, parameter :: ni=ng*ne*nh*ns*na
integer :: zmap(ni,5), id, np, itop, iend

!temp value
double precision :: vtemp
integer :: gcp

double precision :: a1, a2
integer :: ja



! gridpoints per processor ni=ne*nh*ns*na
np=(ni-1)/nproc+1
itop=rank*np+1
iend=MIN((rank+1)*np,ni)

allocate (cmpi(Np),lmpi(Np),ampi(Np),vmpi(Np),medicaidmpi(np),netmpi(Np))
allocate (crecmpi(Np*nproc),lrecmpi(Np*nproc),arecmpi(Np*nproc),vrecmpi(Np*nproc),medicaidrecmpi(np*nproc),netrecmpi(Np*nproc))
allocate (dfmpi(np),dgmpi(np),dmmpi(np))
allocate (dfrecmpi(np*nproc),dgrecmpi(np*nproc),dmrecmpi(np*nproc))


!parallelizing to backout value function
id=0
    do gc=1,ng
    do ec=1,ne
    do hc=1,nh
    do sc=1,ns
    do ac=1,na
    id=id+1
    zmap(id,1)=gc
    zmap(id,2)=ec
    zmap(id,3)=hc
    zmap(id,4)=sc
    zmap(id,5)=ac
    end do
    end do
    end do
    end do
    end do

! Initialize Value Functions and policy functions
Vss=0
lfunss=0
afunss=0

    cfunss=0
!assume agent does not choose foodstamp
g_dfss=1
!assume agent buys private insurance
g_dgss=1
!assume agent does not eligible Medicaid
g_dmss=0
!transfers from Medicaid
medicaidexpfunss=0
!transfers from ACA
netfunss=0
        
    do tc=J,1,-1

        do id=itop,iend
        
        gc=zmap(id,1)
        ec=zmap(id,2)
        hc=zmap(id,3)
        sc=zmap(id,4)
        ac=zmap(id,5)

            if ( hc .gt. sicklevel ) then
            !individual is sick
            sick=1
            else
            sick=0
            end if
	
        economy=economyss(tc)

            if ( economy .eq. 'u' ) then
            
            !do not buy private insurance
            g_dgss(gc,ec,hc,sc,ac,tc)=0
            !do not choose foodstamp
            g_dfss(gc,ec,hc,sc,ac,tc)=1
            !no Medicaid
            g_dmss(gc,ec,hc,sc,ac,tc)=0
            !transfers from ACA
            netfunss(gc,ec,hc,sc,ac,tc)=0

            !if agent works
            work=1
            il=1
            amax=grida(1)
            vss(gc,ec,hc,sc,ac,tc)=-FCNss(amax)
            afunss(gc,ec,hc,sc,ac,tc)=amax
            cfunss(gc,ec,hc,sc,ac,tc)=cmax
            lfunss(gc,ec,hc,sc,ac,tc)=lmax

                do ja=2,na
                            
                amax=grida(ja)
                vtemp=-FCNss(amax)

                    if ( vtemp .gt. vss(gc,ec,hc,sc,ac,tc) ) then
                    vss(gc,ec,hc,sc,ac,tc)=vtemp
                    afunss(gc,ec,hc,sc,ac,tc)=amax
                    cfunss(gc,ec,hc,sc,ac,tc)=cmax
                    lfunss(gc,ec,hc,sc,ac,tc)=lmax
                    end if
                            
                end do
			                    
                do il=2,nl
                do ja=1,na
                amax=grida(ja)
                vtemp=-FCNss(amax)

                    if ( vtemp .gt. vss(gc,ec,hc,sc,ac,tc) ) then
                    vss(gc,ec,hc,sc,ac,tc)=vtemp
                    afunss(gc,ec,hc,sc,ac,tc)=amax
                    cfunss(gc,ec,hc,sc,ac,tc)=cmax
                    lfunss(gc,ec,hc,sc,ac,tc)=lmax
                    end if
                        
                end do
                end do

            !if agent does not work
            work=0	
				
                do ja=1,na
                amax=grida(ja)
                vtemp=-FCNss(amax)
				
                    if ( vtemp .gt. vss(gc,ec,hc,sc,ac,tc) ) then
                    vss(gc,ec,hc,sc,ac,tc)=vtemp
                    afunss(gc,ec,hc,sc,ac,tc)=amax
                    cfunss(gc,ec,hc,sc,ac,tc)=cmax
                    lfunss(gc,ec,hc,sc,ac,tc)=lmax
                    end if
					
                end do
                                    
            end if

            if ( economy .eq. 'm' ) then

                if ( gc .eq. 1 .or. gc .eq. 2 ) then
                
                !if agent works and buys pvt insurance
                buypvt=1
                work=1
                il=1
				
                amax=grida(1)
                vss(gc,ec,hc,sc,ac,tc)=-FCNss(amax)
                afunss(gc,ec,hc,sc,ac,tc)=amax
                cfunss(gc,ec,hc,sc,ac,tc)=cmax
                lfunss(gc,ec,hc,sc,ac,tc)=lmax
                g_dgss(gc,ec,hc,sc,ac,tc)=1
                g_dmss(gc,ec,hc,sc,ac,tc)=elig	
                medicaidexpfunss(gc,ec,hc,sc,ac,tc)=xmed*elig*cov_mdss(tc)
                netfunss(gc,ec,hc,sc,ac,tc)=net
                                
                    do ja=2,na
                    amax=grida(ja)
                    vtemp=-FCNss(amax)                                    

                        if ( vtemp .gt. vss(gc,ec,hc,sc,ac,tc) ) then
                        vss(gc,ec,hc,sc,ac,tc)=vtemp
                        afunss(gc,ec,hc,sc,ac,tc)=amax
                        cfunss(gc,ec,hc,sc,ac,tc)=cmax
                        lfunss(gc,ec,hc,sc,ac,tc)=lmax
                        g_dgss(gc,ec,hc,sc,ac,tc)=1
                        g_dmss(gc,ec,hc,sc,ac,tc)=elig
                        medicaidexpfunss(gc,ec,hc,sc,ac,tc)=xmed*elig*cov_mdss(tc)
                        netfunss(gc,ec,hc,sc,ac,tc)=net					
                        end if
					
                    end do
					
                    do il=2,nl
                    do ja=1,na
                        
                    amax=grida(ja)
                    vtemp=-FCNss(amax)                                    
	
                        if ( vtemp .gt. vss(gc,ec,hc,sc,ac,tc) ) then
                        vss(gc,ec,hc,sc,ac,tc)=vtemp
                        afunss(gc,ec,hc,sc,ac,tc)=amax
                        cfunss(gc,ec,hc,sc,ac,tc)=cmax
                        lfunss(gc,ec,hc,sc,ac,tc)=lmax
                        g_dgss(gc,ec,hc,sc,ac,tc)=1
                        g_dmss(gc,ec,hc,sc,ac,tc)=elig
                        medicaidexpfunss(gc,ec,hc,sc,ac,tc)=xmed*elig*cov_mdss(tc)
                        netfunss(gc,ec,hc,sc,ac,tc)=net
                        end if
							
                    end do							
                    end do

                !if agent does not work and buys pvt insurance
                buypvt=1
                work=0
					
                    do ja=1,na
                    amax=grida(ja)
                    vtemp=-FCNss(amax)                                    
	
                        if ( vtemp .gt. vss(gc,ec,hc,sc,ac,tc) ) then
                        vss(gc,ec,hc,sc,ac,tc)=vtemp
                        afunss(gc,ec,hc,sc,ac,tc)=amax
                        cfunss(gc,ec,hc,sc,ac,tc)=cmax
                        lfunss(gc,ec,hc,sc,ac,tc)=lmax
                        g_dgss(gc,ec,hc,sc,ac,tc)=1
                        g_dmss(gc,ec,hc,sc,ac,tc)=elig
                        medicaidexpfunss(gc,ec,hc,sc,ac,tc)=xmed*elig*cov_mdss(tc)
                        netfunss(gc,ec,hc,sc,ac,tc)=net						
                        end if
    
                    end do
						
                    if ( allbuyss .eq. 0 .or. tc .ge. jr ) then
                    !agent does not buy pvt insurance, does not work
                    buypvt=0
                    work=0
				
                        do ja=1,na
                        amax=grida(ja)
                        vtemp=-FCNss(amax)                                    
	
                            if ( vtemp .gt. vss(gc,ec,hc,sc,ac,tc) ) then
                            vss(gc,ec,hc,sc,ac,tc)=vtemp
                            afunss(gc,ec,hc,sc,ac,tc)=amax
                            cfunss(gc,ec,hc,sc,ac,tc)=cmax
                            lfunss(gc,ec,hc,sc,ac,tc)=lmax
                            g_dgss(gc,ec,hc,sc,ac,tc)=0
                            g_dmss(gc,ec,hc,sc,ac,tc)=elig
                            medicaidexpfunss(gc,ec,hc,sc,ac,tc)=xmed*elig*cov_mdss(tc)
                            netfunss(gc,ec,hc,sc,ac,tc)=net                                        
                            end if
    
                        end do				
					
                    !agent does not buy pvt insurance, but works
                    buypvt=0
                    work=1
	
                        do il=1,nl
                        do ja=1,na
                        amax=grida(ja)
                        vtemp=-FCNss(amax)                                    

                            if ( vtemp .gt. vss(gc,ec,hc,sc,ac,tc) ) then
                            vss(gc,ec,hc,sc,ac,tc)=vtemp
                            afunss(gc,ec,hc,sc,ac,tc)=amax
                            cfunss(gc,ec,hc,sc,ac,tc)=cmax
                            lfunss(gc,ec,hc,sc,ac,tc)=lmax
                            g_dgss(gc,ec,hc,sc,ac,tc)=0
                            g_dmss(gc,ec,hc,sc,ac,tc)=elig
                            medicaidexpfunss(gc,ec,hc,sc,ac,tc)=xmed*elig*cov_mdss(tc)
                            netfunss(gc,ec,hc,sc,ac,tc)=net							
                            end if

                        end do
                        end do
                                                
                    end if
                
                else if ( gc .eq. 3 ) then

                    buypvt=0
                    work=1
                    il=1

                    amax=grida(1)
                    vss(gc,ec,hc,sc,ac,tc)=-FCNss(amax)                                    
                    afunss(gc,ec,hc,sc,ac,tc)=amax
                    cfunss(gc,ec,hc,sc,ac,tc)=cmax
                    lfunss(gc,ec,hc,sc,ac,tc)=lmax
                    g_dgss(gc,ec,hc,sc,ac,tc)=0
                    g_dmss(gc,ec,hc,sc,ac,tc)=elig
                    medicaidexpfunss(gc,ec,hc,sc,ac,tc)=xmed*elig*cov_mdss(tc)
                    netfunss(gc,ec,hc,sc,ac,tc)=net
                                
                        do ja=2,na
                        amax=grida(ja)
                        vtemp=-FCNss(amax)                                    

                            if ( vtemp .gt. vss(gc,ec,hc,sc,ac,tc) ) then
                            vss(gc,ec,hc,sc,ac,tc)=vtemp
                            afunss(gc,ec,hc,sc,ac,tc)=amax
                            cfunss(gc,ec,hc,sc,ac,tc)=cmax
                            lfunss(gc,ec,hc,sc,ac,tc)=lmax
                            g_dgss(gc,ec,hc,sc,ac,tc)=0
                            g_dmss(gc,ec,hc,sc,ac,tc)=elig
                            medicaidexpfunss(gc,ec,hc,sc,ac,tc)=xmed*elig*cov_mdss(tc)
                            netfunss(gc,ec,hc,sc,ac,tc)=net						
                            end if
                        end do
                                            
																	
                        do il=2,nl
                        do ja=1,na
                                            
                        amax=grida(ja)
                        vtemp=-FCNss(amax)                                    
						
                            if ( vtemp .gt. vss(gc,ec,hc,sc,ac,tc) ) then
                            vss(gc,ec,hc,sc,ac,tc)=vtemp	
                            afunss(gc,ec,hc,sc,ac,tc)=amax
                            cfunss(gc,ec,hc,sc,ac,tc)=cmax
                            lfunss(gc,ec,hc,sc,ac,tc)=lmax
                            g_dgss(gc,ec,hc,sc,ac,tc)=0
                            g_dmss(gc,ec,hc,sc,ac,tc)=elig
                            medicaidexpfunss(gc,ec,hc,sc,ac,tc)=xmed*elig*cov_mdss(tc)
                            netfunss(gc,ec,hc,sc,ac,tc)=net
                            end if
							
                        end do
                        end do

                    buypvt=0
                    work=0
										
                        do ja=1,na
                        amax=grida(ja)
                        vtemp=-FCNss(amax)                                    
	
                            if ( vtemp .gt. vss(gc,ec,hc,sc,ac,tc) ) then
                            vss(gc,ec,hc,sc,ac,tc)=vtemp
                            afunss(gc,ec,hc,sc,ac,tc)=amax
                            cfunss(gc,ec,hc,sc,ac,tc)=cmax
                            lfunss(gc,ec,hc,sc,ac,tc)=lmax
                            g_dgss(gc,ec,hc,sc,ac,tc)=0
                            g_dmss(gc,ec,hc,sc,ac,tc)=elig
                            medicaidexpfunss(gc,ec,hc,sc,ac,tc)=xmed*elig*cov_mdss(tc)
                            netfunss(gc,ec,hc,sc,ac,tc)=net						
                            end if
						
                        end do
						
                        if ( tc .ge. jr-1 ) then
                        
                        buypvt=1
                        work=1
						
                            do il=1,nl
                            do ja=1,na
                            amax=grida(ja)
                            vtemp=-FCNss(amax)                                    

                                if ( vtemp .gt. vss(gc,ec,hc,sc,ac,tc) ) then
                                vss(gc,ec,hc,sc,ac,tc)=vtemp
                                afunss(gc,ec,hc,sc,ac,tc)=amax
                                cfunss(gc,ec,hc,sc,ac,tc)=cmax
                                lfunss(gc,ec,hc,sc,ac,tc)=lmax
                                g_dgss(gc,ec,hc,sc,ac,tc)=1
                                g_dmss(gc,ec,hc,sc,ac,tc)=elig
                                medicaidexpfunss(gc,ec,hc,sc,ac,tc)=xmed*elig*cov_mdss(tc)
                                netfunss(gc,ec,hc,sc,ac,tc)=net
                                end if
								
                            end do
                            end do

                        buypvt=1
                        work=0

                            do ja=1,na
                            amax=grida(ja)
                            vtemp=-FCNss(amax)                                    

                                if ( vtemp .gt. vss(gc,ec,hc,sc,ac,tc) ) then
                                vss(gc,ec,hc,sc,ac,tc)=vtemp
                                afunss(gc,ec,hc,sc,ac,tc)=amax
                                cfunss(gc,ec,hc,sc,ac,tc)=cmax
                                lfunss(gc,ec,hc,sc,ac,tc)=lmax
                                g_dgss(gc,ec,hc,sc,ac,tc)=1
                                g_dmss(gc,ec,hc,sc,ac,tc)=elig
                                medicaidexpfunss(gc,ec,hc,sc,ac,tc)=xmed*elig*cov_mdss(tc)
                                netfunss(gc,ec,hc,sc,ac,tc)=net
                                end if
                            end do
                                                    	
                        end if

                end if

            !foodstamps
            
                if ( gc .eq. 1 .or. gc .eq. 2 ) then
                gcp=2
                else if ( gc .eq. 3 ) then
                gcp=3
                end if

                call basefun(grida,na,grida(1),vals,inds)
	
                do ih=1,nh
                    do is=1,ns
                    VM(ih,is)=Vss(gcp,ec,ih,is,1,tc+1)
                    enddo
                temp(ih)=sum(VM(ih,1:ns)*pi(ec,1:ns,sc))
                end do

            vnotpay=U(cmin,zero,phi_h(hc))+bbeta*psi(ec,hc,tc)*sum(temp(1:nh)*pi_h(ec,tc,1:nh,hc))
	
                if ( vnotpay .gt. vss(gc,ec,hc,sc,ac,tc) ) then
                g_dfss(gc,ec,hc,sc,ac,tc)=0
                g_dgss(gc,ec,hc,sc,ac,tc)=0
                afunss(gc,ec,hc,sc,ac,tc)=grida(1)
                cfunss(gc,ec,hc,sc,ac,tc)=cmin
                lfunss(gc,ec,hc,sc,ac,tc)=zero
                vss(gc,ec,hc,sc,ac,tc)=vnotpay
                g_dmss(gc,ec,hc,sc,ac,tc)=0
                medicaidexpfunss(gc,ec,hc,sc,ac,tc)=0
                netfunss(gc,ec,hc,sc,ac,tc)=0
                end if

            end if

            if ( cfunss(gc,ec,hc,sc,ac,tc) .le. zero .or. isnan (cfunss(gc,ec,hc,sc,ac,tc) )  ) then
            print*, 'Consumption negative ,gc,ec,hc,sc,ac,tc '
            print*,cfunss(gc,ec,hc,sc,ac,tc),gc,ec,hc,sc,ac,tc
                call exit
            else if ( (lfunss(gc,ec,hc,sc,ac,tc) .lt. zero) .or. (lfunss(gc,ec,hc,sc,ac,tc) .ge. (lendowment))&
            .or. isnan(lfunss(gc,ec,hc,sc,ac,tc)) ) then
            print*, 'Weird labor!!!', lfunss(gc,ec,hc,sc,ac,tc),gc,ec,hc,sc,ac,tc
                call exit
            end if
            
        !for gathering
        vmpi(id-itop+1)=vss(gc,ec,hc,sc,ac,tc)
        cmpi(id-itop+1)=cfunss(gc,ec,hc,sc,ac,tc)
        lmpi(id-itop+1)=lfunss(gc,ec,hc,sc,ac,tc)
        ampi(id-itop+1)=afunss(gc,ec,hc,sc,ac,tc)
        dfmpi(id-itop+1)=g_dfss(gc,ec,hc,sc,ac,tc)
        dgmpi(id-itop+1)=g_dgss(gc,ec,hc,sc,ac,tc)
        dmmpi(id-itop+1)=g_dmss(gc,ec,hc,sc,ac,tc)
        medicaidmpi(id-itop+1)=medicaidexpfunss(gc,ec,hc,sc,ac,tc)
        netmpi(id-itop+1)=netfunss(gc,ec,hc,sc,ac,tc)

        end do

            call MPI_ALLGATHER (vmpi,np,mpi_double_precision,vrecmpi,np,mpi_double_precision,mpi_comm_world,ierr)
            call MPI_ALLGATHER (cmpi,np,mpi_double_precision,crecmpi,np,mpi_double_precision,mpi_comm_world,ierr)
            call MPI_ALLGATHER (lmpi,np,mpi_double_precision,lrecmpi,np,mpi_double_precision,mpi_comm_world,ierr)
            call MPI_ALLGATHER (ampi,np,mpi_double_precision,arecmpi,np,mpi_double_precision,mpi_comm_world,ierr)
            call MPI_ALLGATHER (dfmpi,np,mpi_integer1,dfrecmpi,np,mpi_integer1,mpi_comm_world,ierr)
            call MPI_ALLGATHER (dgmpi,np,mpi_integer1,dgrecmpi,np,mpi_integer1,mpi_comm_world,ierr)
            call MPI_ALLGATHER (dmmpi,np,mpi_integer1,dmrecmpi,np,mpi_integer1,mpi_comm_world,ierr)
            call MPI_ALLGATHER (medicaidmpi,np,mpi_double_precision,medicaidrecmpi,np,mpi_double_precision,mpi_comm_world,ierr)
            call MPI_ALLGATHER (netmpi,np,mpi_double_precision,netrecmpi,np,mpi_double_precision,mpi_comm_world,ierr)

	id=0
        
            do gc=1,ng
            do ec=1,ne
            do hc=1,nh
            do sc=1,ns
            do ac=1,na
                            
            id=id+1
            vss(gc,ec,hc,sc,ac,tc)=vrecmpi(id)
            cfunss(gc,ec,hc,sc,ac,tc)=crecmpi(id)
            lfunss(gc,ec,hc,sc,ac,tc)=lrecmpi(id)
            afunss(gc,ec,hc,sc,ac,tc)=arecmpi(id)
            g_dfss(gc,ec,hc,sc,ac,tc)=dfrecmpi(id)
            g_dgss(gc,ec,hc,sc,ac,tc)=dgrecmpi(id)
            g_dmss(gc,ec,hc,sc,ac,tc)=dmrecmpi(id)
            medicaidexpfunss(gc,ec,hc,sc,ac,tc)=medicaidrecmpi(id)
            netfunss(gc,ec,hc,sc,ac,tc)=netrecmpi(id)
            end do
            end do
            end do	
            end do
            end do

	end do

deallocate (cmpi,lmpi,ampi,vmpi,medicaidmpi,netmpi)
deallocate (crecmpi,lrecmpi,arecmpi,vrecmpi,medicaidrecmpi,netrecmpi)
deallocate (dfmpi,dgmpi,dmmpi)
deallocate (dfrecmpi,dgrecmpi,dmrecmpi)

        

        
end subroutine housess
