// Compute Wage Per Hour by Education
clear
set obs 162
seq aget, f(20) t(100) b(2)
seq edcat, f(1) t(2) b(1)
compress
save predictsample, replace

*******************************************************************************
use 02_meps_prep_02.dta, clear
keep dupersid dobyy sex begrfy1 hrwgt* thrwgt* hrwkt* wagepy* combwaget* highcost* edcat* 

cap drop id
	egen id = group(dupersid dobyy sex begrfy1)

reshape long aget ///
		hrwgt thrwgt hrwkt wagepy combwaget highcost ///
		, i(id) j(year)
		
* Minimum Wage from here: https://www.dol.gov/whd/minwage/chart.htm
cap drop minwage
gen minwage=.
replace minwage = 4.75 if inrange(begrfy1, 1996, 1996)
replace minwage = 5.15 if inrange(begrfy1, 1997, 2006)
replace minwage = 5.85 if inrange(begrfy1, 2007, 2007)
replace minwage = 6.55 if inrange(begrfy1, 2008, 2008)
replace minwage = 7.25 if inrange(begrfy1, 2009, 2016)

cap drop lnwage 
gen lnwage = . 
replace lnwage = ln(hrwgt + 1) if thrwgt>=0.5*minwage
/*2240 observations have wage less than half of minumum wage*/
cap drop plwage
cap drop wage
forvalues x = 2/4 {
cap drop age`x'
gen age`x' = aget^`x'
}
reg lnwage c.aget##i.edcat c.age2##i.edcat c.age3##i.edcat c.age4##i.edcat, vce(cluster id)
use predictsample, clear
forvalues x = 2/4 {
cap drop age`x'
gen age`x' = aget^`x'
}
predict plwage
gen wage = exp(plwage)
twoway (line wage aget if edcat==1, sort(aget) lpattern(solid) lcolor(gs10)) ///
		(line wage aget if edcat==2, sort(aget) lpattern(dash) ) ///
		, legend(label(1 "No-College") label(2 "College") row(2)) ///
		ytitle("Wage per Hour (USD2010)") xtitle("Age")
drop plwage
sort aget edcat
drop age2 age3 age4
reshape wide wage, i(aget) j(edcat)
gen nc = wage1 if _n==1
gen c = wage2 if _n==1
carryforward nc, carryalong(c) replace
gen normnon = wage1/nc
gen normcol = wage2/c
export excel using Wage.xls, sheet("WagebyEd") sheetreplace firstrow(variables)
